/// <reference types="node" />
import type { ReadonlyDeep } from 'type-fest';
import * as Native from '../Native';
import { Wrapper } from '../Native';
export declare enum Environment {
    Staging = 0,
    Production = 1
}
export type ServiceAuth = {
    username: string;
    password: string;
};
export type CDSRequestOptionsType = {
    e164s: Array<string>;
    acisAndAccessKeys: Array<{
        aci: string;
        accessKey: string;
    }>;
    returnAcisWithoutUaks: boolean;
    abortSignal?: AbortSignal;
};
export type CDSResponseEntryType<Aci, Pni> = {
    aci: Aci | undefined;
    pni: Pni | undefined;
};
export type CDSResponseEntries<Aci, Pni> = Map<string, CDSResponseEntryType<Aci, Pni>>;
export interface CDSResponseType<Aci, Pni> {
    entries: CDSResponseEntries<Aci, Pni>;
    debugPermitsUsed: number;
}
export type ChatRequest = Readonly<{
    verb: string;
    path: string;
    headers: ReadonlyArray<[string, string]>;
    body?: Uint8Array;
    timeoutMillis?: number;
}>;
type ConnectionManager = Wrapper<Native.ConnectionManager>;
/** Low-level async runtime control, mostly just exported for testing. */
export declare class TokioAsyncContext {
    readonly _nativeHandle: Native.TokioAsyncContext;
    constructor(handle: Native.TokioAsyncContext);
    makeCancellable<T>(abortSignal: AbortSignal | undefined, promise: Promise<T>): Promise<T>;
}
/**
 * Provides API methods to connect and communicate with the Chat Service.
 * Before using either authenticated or unauthenticated channels,
 * a corresponding `connect*` method must be called.
 * It's also important to call {@link #disconnect()} method when the instance is no longer needed.
 */
export declare class ChatService {
    private readonly asyncContext;
    private readonly chatService;
    constructor(asyncContext: TokioAsyncContext, connectionManager: ConnectionManager);
    /**
     * Initiates termination of the underlying connection to the Chat Service. After the service is
     * disconnected, it will not attempt to automatically reconnect until you call
     * {@link #connectAuthenticated()} and/or {@link #connectUnauthenticated()}.
     *
     * Note: the same instance of `ChatService` can be reused after `disconnect()` was
     * called.
     */
    disconnect(): Promise<void>;
    /**
     * Initiates establishing of the underlying unauthenticated connection to the Chat Service. Once
     * the service is connected, all the requests will be using the established connection. Also, if
     * the connection is lost for any reason other than the call to {@link #disconnect()}, an
     * automatic reconnect attempt will be made.
     *
     * @throws {AppExpiredError} if the current app version is too old (as judged by the server).
     * @throws {LibSignalError} with other codes for other failures.
     */
    connectUnauthenticated(options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ChatServiceDebugInfo>;
    /**
     * Initiates establishing of the underlying authenticated connection to the Chat Service. Once the
     * service is connected, all the requests will be using the established connection. Also, if the
     * connection is lost for any reason other than the call to {@link #disconnect()}, an automatic
     * reconnect attempt will be made.
     *
     * Calling this method will result in starting to accept incoming requests from the Chat Service.
     *
     * @throws {AppExpiredError} if the current app version is too old (as judged by the server).
     * @throws {DeviceDelinkedError} if the current device has been delinked.
     * @throws {LibSignalError} with other codes for other failures.
     */
    connectAuthenticated(options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ChatServiceDebugInfo>;
    /**
     * Sends request to the Chat Service over an unauthenticated channel.
     *
     * In addition to the response, an object containing debug information about the request flow is
     * returned.
     *
     * @throws {ChatServiceInactive} if you haven't called {@link #connectUnauthenticated()} (as a
     * rejection of the promise).
     */
    unauthenticatedFetchAndDebug(chatRequest: ChatRequest, options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ResponseAndDebugInfo>;
    /**
     * Sends request to the Chat Service over an unauthenticated channel.
     *
     * @throws {ChatServiceInactive} if you haven't called {@link #connectUnauthenticated()} (as a
     * rejection of the promise).
     */
    unauthenticatedFetch(chatRequest: ChatRequest, options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ChatResponse>;
    /**
     * Sends request to the Chat Service over an authenticated channel.
     *
     * In addition to the response, an object containing debug information about the request flow is
     * returned.
     *
     * @throws {ChatServiceInactive} if you haven't called {@link #connectAuthenticated()} (as a
     * rejection of the promise).
     */
    authenticatedFetchAndDebug(chatRequest: ChatRequest, options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ResponseAndDebugInfo>;
    /**
     * Sends request to the Chat Service over an authenticated channel.
     *
     * @throws {ChatServiceInactive} if you haven't called {@link #connectAuthenticated()} (as a
     * rejection of the promise).
     */
    authenticatedFetch(chatRequest: ChatRequest, options?: {
        abortSignal?: AbortSignal;
    }): Promise<Native.ChatResponse>;
    static buildHttpRequest(chatRequest: ChatRequest): {
        _nativeHandle: Native.HttpRequest;
    };
}
export declare class Net {
    private readonly asyncContext;
    private readonly connectionManager;
    /**
     * Instance of the {@link Svr3Client} to access SVR3.
     */
    svr3: Svr3Client;
    constructor(env: Environment, userAgent: string);
    /**
     * Creates a new instance of {@link ChatService}.
     */
    newChatService(): ChatService;
    /**
     * Enables/disables IPv6 for all new connections (until changed).
     *
     * The flag is `true` by default.
     */
    setIpv6Enabled(ipv6Enabled: boolean): void;
    /**
     * Sets the proxy host to be used for all new connections (until overridden).
     *
     * Sets a domain name and port to be used to proxy all new outgoing
     * connections. The proxy can be overridden by calling this method again or
     * unset by calling {@link #clearProxy}.
     *
     * Throws if the host or port is structurally invalid, such as a port that doesn't fit in u16.
     */
    setProxy(host: string, port: number): void;
    /**
     * Ensures that future connections will be made directly, not through a proxy.
     *
     * Clears any proxy configuration set via {@link #setProxy}. If none was set, calling this
     * method is a no-op.
     */
    clearProxy(): void;
    cdsiLookup({ username, password }: Readonly<ServiceAuth>, { e164s, acisAndAccessKeys, returnAcisWithoutUaks, abortSignal, }: ReadonlyDeep<CDSRequestOptionsType>): Promise<CDSResponseType<string, string>>;
}
/**
 * This interface provides functionality for communicating with SVR3
 *
 * Its instance can be obtained from an {@link Net#svr3} property
 * of the {@link Net} class.
 *
 * Example usage:
 *
 * @example
 * ```ts
 * import { Environment, Net } from '../net';
 * // Obtain an instance
 * const SVR3 = new Net(Environment.Staging).svr3;
 * // Instantiate ServiceAuth with the username and password obtained from the Chat Server.
 * const auth = { username: USERNAME, password: ENCLAVE_PASSWORD };
 * // Store a value in SVR3. Here 10 is the number of permitted restore attempts.
 * const shareSet = await SVR3.backup(SECRET_TO_BE_STORED, PASSWORD, 10, auth);
 * const restoredSecret = await SVR3.restore( PASSWORD, shareSet, auth);
 * ```
 */
export interface Svr3Client {
    /**
     * Backup a secret to SVR3.
     *
     * Error messages are log-safe and do not contain any sensitive data.
     *
     * @param what - The secret to be stored. Must be 32 bytes long.
     * @param password - User-provided password that will be used to derive the
     * encryption key for the secret.
     * @param maxTries - Number of times the secret will be allowed to be guessed.
     * Each call to {@link Svr3Client#restore} that has reached the server will
     * decrement the counter. Must be positive.
     * @param auth - An instance of {@link ServiceAuth} containing the username
     * and password obtained from the Chat Server. The password is an OTP which is
     * generally good for about 15 minutes, therefore it can be reused for the
     * subsequent calls to either backup or restore that are not too far apart in
     * time.
     * @returns A `Promise` which--when awaited--will return a byte array with a
     * serialized masked share set. It is supposed to be an opaque blob for the
     * clients and therefore no assumptions should be made about its contents.
     * This byte array should be stored by the clients and used to restore the
     * secret along with the password. Please note that masked share set does not
     * have to be treated as secret.
     *
     * The returned `Promise` can also fail due to the network issues (including a
     * connection timeout), problems establishing the Noise connection to the
     * enclaves, or invalid arguments' values. {@link IoError} errors can, in
     * general, be retried, although there is already a retry-with-backoff
     * mechanism inside libsignal used to connect to the SVR3 servers. Other
     * exceptions are caused by the bad input or data missing on the server. They
     * are therefore non-actionable and are guaranteed to be thrown again when
     * retried.
     */
    backup(what: Buffer, password: string, maxTries: number, auth: Readonly<ServiceAuth>, options?: {
        abortSignal?: AbortSignal;
    }): Promise<Buffer>;
    /**
     * Restore a secret from SVR3.
     *
     * Error messages are log-safe and do not contain any sensitive data.
     *
     * @param password - User-provided password that will be used to derive the
     * decryption key for the secret.
     * @param shareSet - a serialized masked share set returned by a call to
     * {@link Svr3Client#backup}.
     * @param auth - An instance of {@link ServiceAuth} containing the username
     * and password obtained from the Chat Server. The password is an OTP which is
     * generally good for about 15 minutes, therefore it can be reused for the
     * subsequent calls to either backup or restore that are not too far apart in
     * time.
     * @returns A `Promise` which--when awaited--will return a
     * {@link RestoredSecret} object, containing the restored secret.
     *
     * The returned `Promise` can also fail due to the network issues (including
     * the connection timeout), problems establishing the Noise connection to the
     * enclaves, or invalid arguments' values. {@link IoError} errors can, in
     * general, be retried, although there is already a retry-with-backoff
     * mechanism inside libsignal used to connect to the SVR3 servers. Other
     * exceptions are caused by the bad input or data missing on the server. They
     * are therefore non-actionable and are guaranteed to be thrown again when
     * retried.
     *
     * - {@link SvrDataMissingError} is returned when the maximum restore attempts
     * number has been exceeded or if the value has never been backed up.
     * - {@link SvrRestoreFailedError} is returned when the combination of the
     * password and masked share set does not result in successful restoration
     * of the secret.
     * - {@link SvrRequestFailedError} is returned when the de-serialization of a
     * masked share set fails, or when the server requests fail for reasons
     * other than "maximum attempts exceeded".
     */
    restore(password: string, shareSet: Buffer, auth: Readonly<ServiceAuth>, options?: {
        abortSignal?: AbortSignal;
    }): Promise<RestoredSecret>;
    /**
     * Remove a value stored in SVR3.
     *
     * This method will succeed even if the data has never been backed up in the
     * first place.
     *
     * Error messages are log-safe and do not contain any sensitive data.
     *
     * @param auth - An instance of {@link ServiceAuth} containing the username
     * and password obtained from the Chat Server. The password is an OTP which is
     * generally good for about 15 minutes, therefore it can be reused for the
     * subsequent calls to either backup or restore that are not too far apart in
     * time.
     * @returns A `Promise` successful completion of which will mean the data has
     * been removed.
     *
     * The returned `Promise` can also fail due to the network issues (including
     * the connection timeout), problems establishing the Noise connection to the
     * enclaves, or invalid arguments' values. {@link IoError} errors can, in
     * general, be retried, although there is already a retry-with-backoff
     * mechanism inside libsignal used to connect to the SVR3 servers. Other
     * exceptions are caused by the bad input or data missing on the server. They
     * are therefore non-actionable and are guaranteed to be thrown again when
     * retried.
     */
    remove(auth: Readonly<ServiceAuth>, options?: {
        abortSignal?: AbortSignal;
    }): Promise<void>;
}
/**
 * A simple data class containing the secret restored from SVR3 as well as the
 * number of restore attempts remaining.
 */
export declare class RestoredSecret {
    readonly triesRemaining: number;
    readonly value: Buffer;
    constructor(serialized: Buffer);
}
export {};
